@@ -275,6 +275,8 @@ def compile_macros(macros):
               help='Path to the directory containing the source file for the macro.')
 @click.option('-n', '--name',
               help='Name of the macro. By default, the name of the input directory is used.')
+@click.option('-f', '--force', is_flag=True,
+              help='Force overwrite existing files.')
 @click.option('--copy-files/--do-not-copy-files', 'copy_files', default=True,
               help='Whether to copy files from the input directory (required if not already copied).')
 @click.option('--add-to-workspace-extension/--do-not-add-to-workspace-extension', 'workspace_extension',
@@ -290,6 +292,8 @@ def add_macro(**kwargs):
 @click.command(name='remove_macro')
 @click.option('-n', '--name', required=True,
               help='Name of the macro.')
+@click.option('-f', '--force', is_flag=True,
+              help='Force remove files without notification.')
 @click.option('--remove-files/--do-not-remove-files', 'remove_files',
               default=False, show_default=True,
               help='Whether to remove the macro from the workspace extension list only or also remove the files.')
@@ -365,6 +369,8 @@ def harmonize_np(ws_files, reference, input_config_path, base_path, outfile):
 @click.option('-c', '--configuration', default=None,
               help='Path to the json configuration file containing'
                    ' the minimizer options for NP profiling.')
+@click.option('--rebuild/--do-not-rebuild', default=False, show_default=True,
+              help='Rebuild the workspace.')
 @click.option('-v', '--verbosity', default='INFO', show_default=True,
               type=click.Choice(["DEBUG", "INFO", "WARNING", "ERROR"], case_sensitive=False),
               help='Verbosity level.')
@@ -382,6 +388,7 @@ def generate_asimov(**kwargs):
     profile_param = kwargs.pop('profile_param')
     snapshot_name = kwargs.pop('snapshot_name')
     config_file   = kwargs.pop('configuration')
+    rebuild       = kwargs.pop('rebuild')
     if config_file is not None:
         config = json.load(open(config_file, 'r'))
     else:
@@ -403,7 +410,7 @@ def generate_asimov(**kwargs):
     analysis = AnalysisBase(filename, poi_name=poi_name, data_name=data_name,
                             config=config, verbosity=verbosity)
     analysis.generate_asimov(**asimov_config)
-    analysis.save(outname)
+    analysis.save(outname, rebuild=rebuild)
 
 @click.command(name='generate_standard_asimov')
 @click.option('-i', '--input_file', 'filename', required=True, 
@@ -423,6 +430,8 @@ def generate_asimov(**kwargs):
               help='Parameters to profile')
 @click.option('-s', '--snapshot', 'snapshot_name', default=None, show_default=True,
               help='Name of initial snapshot')
+@click.option('--rebuild/--do-not-rebuild', default=False, show_default=True,
+              help='Rebuild the workspace.')
 @click.option('--asimov_names', default=None, show_default=True,
               help='Names of the output asimov datasets (separated by commas). If not specified, '
                    'a default name for the corresponding asimov type will be given.')
@@ -457,6 +466,7 @@ def generate_standard_asimov(**kwargs):
     asimov_names = kwargs.pop("asimov_names")
     asimov_snapshots = kwargs.pop("asimov_snapshots")
     verbosity = kwargs.pop("verbosity")
+    rebuild = kwargs.pop("rebuild")
     config = {
         'fix_param': fix_param,
         'profile_param': profile_param,
@@ -470,7 +480,7 @@ def generate_standard_asimov(**kwargs):
     generator.generate_standard_asimov(asimov_types, poi_scale=poi_scale,
                                        asimov_names=asimov_names,
                                        asimov_snapshots=asimov_snapshots)
-    generator.save(outname)
+    generator.save(outname, rebuild=rebuild)
 
 @click.command(name='toy_significance')
 @click.option('-i', '--input_file', 'filename', required=True, 
