@@ -18,6 +18,8 @@ import click
 @click.option('--save_snapshot', default=None, show_default=True,
               help='Save fitted values of all variables as a snapshot and restore all variables to '
               'their initial values. Should be used together with --save_ws.')
+@click.option('--rebuild/--no-rebuild', default=True, show_default=True,
+              help='Save fitted workspace by rebuilding it. Should be used together with --save_ws.')
 @click.option('--outdir', default="pulls", show_default=True,
               help='Output directory for pulls output.')
 @click.option('--export_as_np_pulls/--skip_export_as_np_pulls', default=False, show_default=True,
@@ -85,6 +87,7 @@ def likelihood_fit(**kwargs):
     Perform likelihood fit on a workspace
     """
     do_minos = kwargs.pop("minos")
+    rebuild = kwargs.pop("rebuild")
     pois = [poi for poi in kwargs.pop("pois").split(",") if poi]
     _kwargs = {}
     for arg_name in ["outname", "save_log", "display", "save_result",
@@ -170,7 +173,7 @@ def likelihood_fit(**kwargs):
             from quickstats.components.basics import WSArgument
             analysis.save_snapshot(snapshot_name, WSArgument.MUTABLE)
             analysis.load_snapshot(analysis.kInitialSnapshotName)
-        analysis.save(filename)
+        analysis.save(filename, rebuild=rebuild)
         
 @click.command(name='np_correlation')
 @click.option('-i', '--input_file', "filename", required=True, 
