@@ -149,7 +149,7 @@ class NuisanceParameterPull(object):
             'minimizer_type'   : minimizer_type,
             'minimizer_algo'   : minimizer_algo,
             'default_strategy' : strategy,
-            'opt_const'        : optimize,
+            'optimize'         : optimize,
             'precision'        : precision,
             'eps'              : eps,
             'retry'            : retry,
