@@ -9,7 +9,8 @@ from .builtin_methods import BUILTIN_METHODS
 from .actions import *
 from .roo_config_parser import RooConfigParser
 
-from quickstats import Timer, AbstractObject
+from quickstats import Timer, AbstractObject, PathManager
+from quickstats.interface.root import TFile
 from quickstats.utils.root_utils import declare_expression, close_all_root_files
 
 class RooProcessor(AbstractObject):
@@ -89,7 +90,15 @@ class RooProcessor(AbstractObject):
                 all_files += glob.glob(fname)
         if not all_files:
             raise FileNotFoundError(f'file "{filename}" does not exist')
-        return all_files
+        PathManager.check_files(all_files)
+        valid_files = []
+        for file in all_files:
+            rf = ROOT.TFile(file)
+            if TFile.is_corrupt(rf):
+                self.stdout.warning(f'WARNING: Found empty/corrupted file "{file}". Skipped.')
+            else:
+                valid_files.append(file)
+        return valid_files
     
     def run_action(self, action:RooProcBaseAction):
         if not self.rdf:
