@@ -171,13 +171,19 @@ class ToyLimitCalculator(AbstractObject):
         interval = (scan_max - scan_min) / (steps - 1)
         for i in range(steps):
             mu = scan_min + i * interval
-            self.poi.setVal(mu)
-            test_statistic = self.plr.EvaluateProfileLikelihood(0, 
-                                                                self.model_base.data,
-                                                                ROOT.RooArgSet(self.poi))
-            self.stdout.info("INFO: mu = {}, test statistic = {}".format(mu, test_statistic))
-            if test_statistic < 0:
-                raise RuntimeError("test statistic failed to converge")
+            self._check_test_statistics(mu)
+    
+    def _check_test_statistics(self, mu:float):
+        mu_0  = self.poi.getVal()
+        self.poi.setVal(mu)
+        test_statistic = self.plr.EvaluateProfileLikelihood(0, 
+                                                            self.model_base.data,
+                                                            ROOT.RooArgSet(self.poi))
+        self.stdout.info(f"INFO: mu = {mu}, test statistic = {test_statistic}")
+        if test_statistic < 0:
+            raise RuntimeError("test statistic failed to converge")
+        self.poi.setVal(mu_0)
+        return True        
                 
     def append(self, result: Union[ROOT.RooStats.HypoTestInverterResult,
                                    ROOT.RooStats.HypoTestResult]):
