@@ -17,8 +17,7 @@ import ROOT
 import quickstats
 from quickstats import semistaticmethod, Timer, AbstractObject, GeneralEnum
 from quickstats.components import ExtendedModel
-from quickstats.utils.io import Verbosity
-from quickstats.utils.root_utils import load_macro, get_macro_dir
+from quickstats.utils.xml_tools import TXMLTree
 from quickstats.utils.common_utils import format_delimiter_enclosed_text, remove_list_duplicates
 from quickstats.maths.numerics import is_float, pretty_value
 from quickstats.components.basics import WSArgument
@@ -91,6 +90,8 @@ class XMLWSModifier(XMLWSBase):
             "snapshot_globs"     : source.pop("snapshot_globs", []),
             "snapshot_pois"      : source.pop("snapshot_pois", []),
             "snapshot_all"       : source.pop("snapshot_all", []),
+            "snapshot_list"      : source.pop("snapshot_list", []),
+            "dataset_list"       : source.pop("dataset_list", None),
             "fix_parameters"     : source.pop("fix_parameters", None),
             "profile_parameters" : source.pop("profile_parameters", None),
             "strict"             : source.pop("strict", True)
@@ -152,7 +153,7 @@ class XMLWSModifier(XMLWSBase):
         self.parse_config_dict(source)        
     
     def parse_config_xml(self, filename:str):
-        root = self._xml_to_dict(filename)
+        root = TXMLTree.load_as_dict(filename)
         config = {
             'input_file'       : self._get_node_attrib(root, "InFile", required=True),
             'output_file'      : self._get_node_attrib(root, "OutFile", required=True),
@@ -165,6 +166,8 @@ class XMLWSModifier(XMLWSBase):
             'snapshot_globs'   : self._get_node_attrib(root, "SnapshotGO", required=False, default=[], dtype="str_list"),
             'snapshot_pois'    : self._get_node_attrib(root, "SnapshotPOI", required=False, default=[], dtype="str_list"),
             'snapshot_all'     : self._get_node_attrib(root, "SnapshotAll", required=False, default=[], dtype="str_list"),
+            'snapshot_list'    : self._get_node_attrib(root, "SnapshotList", required=False, default=[], dtype="str_list"),
+            'dataset_list'     : self._get_node_attrib(root, "DatasetList", required=False, default=None, dtype="str_list"),
             "fix_parameters"     : self._get_node_attrib(root, "FixParameters", required=False, default=None),
             "profile_parameters" : self._get_node_attrib(root, "ProfileParameters", required=False, default=None),
             'strict'           : self._get_node_attrib(root, "Strict", required=False, default="true", dtype="bool")
@@ -264,7 +267,8 @@ class XMLWSModifier(XMLWSBase):
         if len(self.actions['rename']['model_config']) > 1:
             raise RuntimeError("model config is renamed more than once")
     
-    def create_modified_workspace(self, infile:Optional[str]=None, outfile:Optional[str]=None):
+    def create_modified_workspace(self, infile:Optional[str]=None, outfile:Optional[str]=None,
+                                  import_class_code:bool=True, recreate:bool=True):
         with Timer() as t:
             self.sanity_check()
             # override path to the input workspace
@@ -300,9 +304,10 @@ class XMLWSModifier(XMLWSBase):
             # nothing is done
             if not flag:
                 self.stdout.info("INFO: No new objects are defined.")
-
+            
+            dataset_list = self.config['dataset_list']
             # need to import datasets first so the variables are also imported into the workspace
-            self.import_datasets(ws_orig, ws_tmp)
+            self.import_datasets(ws_orig, ws_tmp, dataset_list)
 
             title_str = format_delimiter_enclosed_text("Step 3: Rename variables", "-")
             self.stdout.info(title_str)
@@ -320,7 +325,7 @@ class XMLWSModifier(XMLWSBase):
                 ws_final = ROOT.RooWorkspace(ws_name)
                 sim_pdf = self.remake_simultaneous_pdf(mc_orig, ws_tmp)
                 self.import_object(ws_final, sim_pdf)
-                self.import_datasets(ws_orig, ws_final)
+                self.import_datasets(ws_orig, ws_final, dataset_list)
             else:
                 ws_final = ws_tmp        
 
@@ -332,9 +337,12 @@ class XMLWSModifier(XMLWSBase):
             self.rename_objects(ws_final, mc_final)
 
             self.import_object(ws_final, mc_final, silent=False)
-            self.import_class_code(ws_final)
+            if import_class_code:
+                self.import_class_code(ws_final)
 
             self.create_snapshots(ws_orig, mc_orig, ws_final, mc_final, rename_map)
+            
+            self.create_generic_objects(ws_orig, ws_final, rename_map)
 
             self.generate_asimov(ws_final)
 
@@ -343,8 +351,9 @@ class XMLWSModifier(XMLWSBase):
             outdir  = os.path.dirname(outfile)
             if outdir and (not os.path.exists(outdir)):
                 os.makedirs(outdir)
-            ws_final.writeToFile(outfile, True)
-            self.stdout.info(f"INFO: Saved output workspace as \"{outfile}\".")
+
+            ws_final.writeToFile(outfile, recreate)
+            self.stdout.info(f'INFO: Saved output workspace as "{outfile}".')
         self.stdout.info(f"INFO: Total time taken: {t.interval:.3f}s")
     
     def post_process_workspace(self, ws:ROOT.RooWorkspace):
@@ -371,21 +380,6 @@ class XMLWSModifier(XMLWSBase):
             model.fix_parameters(self.config["fix_parameters"])
         if self.config["profile_parameters"] is not None:
             model.profile_parameters(self.config["profile_parameters"])
-    
-    def import_class_code(self, ws:ROOT.RooWorkspace):
-        all_function_class = [i.ClassName().split("::")[-1] for i in ws.allFunctions()]
-        all_pdf_class = [i.ClassName().split("::")[-1] for i in ws.allPdfs()]
-        all_class = all_function_class + all_pdf_class
-        for extension, cls in self.custom_classes.items():
-            # the workspace does not use the extended class, no need to import
-            if extension not in all_class:
-                continue
-            macro_dir = get_macro_dir(extension)
-            ws.addClassDeclImportDir(macro_dir)
-            ws.addClassImplImportDir(macro_dir)
-            ws.importClassCode(cls)
-            self.stdout.info(f"INFO: Imported class code for \"{cls.GetName()}\".")
-        ws.importClassCode()
 
     def generate_asimov(self, ws:ROOT.RooWorkspace):
         n_asimov = len(self.actions['asimov'])
@@ -396,40 +390,55 @@ class XMLWSModifier(XMLWSBase):
                                        minimizer_config=self.minimizer_config)
         for attributes in self.actions['asimov']:
             asimov_handler.generate_single_asimov(attributes)
-        
-    def copy_snapshot_variables(self, old_ws:ROOT.RooWorkspace, old_mc:ROOT.RooStats.ModelConfig,
-                                new_ws:ROOT.RooWorkspace, new_mc:ROOT.RooStats.ModelConfig,
-                                snapshot_name:str, variable_type:Union[WSArgument, str],
-                                rename_map:Optional[Dict]=None):
-        snapshot = old_ws.loadSnapshot(snapshot_name)
-        if not snapshot:
-            self.stdout.warning(f"WARNING: Snapshot {snapshot_name} not found in the original workspace")
-        # get variables from orginal workspace
-        variables = self.get_variables(old_mc, variable_type)
+            
+    def copy_variable(self, old_variable:ROOT.RooRealVar, new_variable:ROOT.RooRealVar):
+        copied_variable = new_variable.Clone()
+        copied_variable.setVal(old_variable.getVal())
+        copied_variable.setRange(old_variable.getMin(), old_variable.getMax())
+        copied_variable.setConstant(old_variable.isConstant())
+        copied_variable.setError(old_variable.getError())
+        return copied_variable
+            
+    def copy_variables(self, variables:ROOT.RooArgSet, new_ws:ROOT.RooWorkspace, rename_map:Optional[Dict]=None):
         if rename_map is None:
             rename_map = {}
-        self.stdout.info(f"INFO: Copying snapshot \"{snapshot_name}\" from the original workspace "
-                         "to the new workspace")
         renamed_variables_old = ROOT.RooArgSet()
         renamed_variables_new = ROOT.RooArgSet()
         for variable in variables:
             name = variable.GetName()
             if name in rename_map:
                 new_name = rename_map[name]
-                new_variable = new_ws.var(name)
+                # note only renaming of RooRealVar instance will be considered
+                new_variable = new_ws.var(new_name)
                 if not new_variable:
-                    self.stdout.warning(f"WARNING: The snapshot variable \"{name}\" not found in the new workspace. Skipped.")
+                    self.stdout.warning(f'WARNING: The argset variable "{name}" not found in the new workspace. Skipped.')
                 # avoid modifying the original value
-                _new_variable = new_variable.Clone()
-                _new_variable.setVal(variable.getVal())
-                _new_variable.setRange(variable.getMin(), variable.getMax())
-                _new_variable.setConstant(variable.isConstant())
-                _new_variable.setError(variable.getError())
+                copied_variable = self.copy_variable(variable, new_variable)
                 renamed_variables_old.add(variable)
-                renamed_variables_new.add(_new_variable)
+                renamed_variables_new.add(copied_variable)
         variables.remove(renamed_variables_old)
         variables.add(renamed_variables_new)
-        new_mc.saveSnapshot(snapshot_name, variables)
+                               
+    def copy_snapshot_variables(self, old_ws:ROOT.RooWorkspace, old_mc:ROOT.RooStats.ModelConfig,
+                                new_ws:ROOT.RooWorkspace, new_mc:ROOT.RooStats.ModelConfig,
+                                snapshot_name:str, variable_type:Optional[Union[WSArgument, str]]=None,
+                                rename_map:Optional[Dict]=None):
+        snapshot = old_ws.getSnapshot(snapshot_name)
+        if not snapshot:
+            self.stdout.warning(f"WARNING: Snapshot {snapshot_name} not found in the original workspace")
+        # get variables from orginal workspace, make sure to make a clone
+        if variable_type is None:
+            variables = snapshot.Clone()
+        else:
+            target_variables = self.get_variables(old_mc, variable_type)
+            variables = snapshot.Clone().selectCommon(target_variables)
+        if rename_map is None:
+            rename_map = {}
+        self.stdout.info(f"INFO: Copying snapshot \"{snapshot_name}\" from the original workspace "
+                         "to the new workspace")
+        self.copy_variables(variables, new_ws, rename_map)
+        # important: the third argument decide whether to take values from the argset or from the workspace
+        new_ws.saveSnapshot(snapshot_name, variables, True)
         
     def get_variables(self, mc:ROOT.RooStats.ModelConfig, variable_type:Union[WSArgument, str]):
         _variable_type = WSArgument.parse(variable_type)
@@ -471,7 +480,52 @@ class XMLWSModifier(XMLWSBase):
         for vars_snapshot_name in vars_snapshot_names:
             self.copy_snapshot_variables(old_ws, old_mc, new_ws, new_mc, vars_snapshot_name,
                                          WSArgument.CORE, rename_map)
-        
+        # any variables
+        vars_snapshot_names = self.config['snapshot_list']
+        for vars_snapshot_name in vars_snapshot_names:
+            self.copy_snapshot_variables(old_ws, old_mc, new_ws, new_mc, vars_snapshot_name,
+                                         None, rename_map)
+            
+    def create_generic_objects(self, old_ws:ROOT.RooWorkspace, new_ws:ROOT.RooWorkspace,
+                               rename_map:Optional[Dict]=None):
+        generic_objects = old_ws.allGenericObjects()
+        # only create objects of type RooArgSet
+        generic_objects = [obj for obj in generic_objects if isinstance(obj, ROOT.RooArgSet)]
+        if rename_map is None:
+            rename_map = {}
+        for generic_object in generic_objects:
+            name = generic_object.GetName()
+            if not name:
+                self.stdout.warning('WARNING: Found generic object without name. '
+                                    'The object will not be imported to the new workspace')
+                continue
+            self.stdout.info(f'INFO: Copying generic object "{name}" from the original workspace to the new workspace"')
+            if all(isinstance(obj, ROOT.RooRealVar) for obj in generic_object):
+                argset = generic_object.Clone()
+                self.copy_variables(argset, new_ws, rename_map)
+            else:
+                argset = self.recreate_argset(generic_object, new_ws, rename_map)
+            new_ws.Import(argset, name, True)
+            
+            
+    def recreate_argset(self, argset:ROOT.RooArgSet, ws:ROOT.RooWorkspace, rename_map:Optional[Dict]=None):
+        argset_name = argset.GetName()
+        new_argset = ROOT.RooArgSet()
+        new_argset.setName(argset_name)
+        for arg in argset:
+            name = arg.GetName()
+            name = rename_map.get(name, name)
+            new_arg = ws.obj(name)
+            if not new_arg:
+                raise RuntimeError(f'WARNING: The variable "{name}" from the argset "{argset_name}" '
+                                   f'not found in the new workspace.')
+            if isinstance(new_arg, ROOT.RooRealVar):
+                copied_arg = self.copy_variable(arg, new_arg)
+                new_argset.add(copied_arg)
+            else:
+                new_argset.add(new_arg)
+        return new_argset
+ 
     def create_model_config(self, old_mc:ROOT.RooStats.ModelConfig, new_ws:ROOT.RooWorkspace,
                             rename_map:Optional[Dict]=None):
         if rename_map is None:
@@ -651,8 +705,17 @@ class XMLWSModifier(XMLWSBase):
                 rename_table = "\n".join(self.get_name_mapping_str_arrays(rename_map))
                 self.stdout.info(rename_table)                
         
-    def import_datasets(self, old_ws:ROOT.RooWorkspace, new_ws:ROOT.RooWorkspace):
-        datasets = old_ws.allData()
+    def import_datasets(self, old_ws:ROOT.RooWorkspace, new_ws:ROOT.RooWorkspace,
+                        dataset_names:Optional[List[str]]=None):
+        if dataset_names is None:
+            datasets = old_ws.allData()
+        else:
+            datasets = []
+            for dataset_name in dataset_names:
+                dataset = old_ws.data(dataset_name)
+                if not dataset:
+                    raise RuntimeError(f"dataset {dataset_name} does not exist in the original workspace")
+                datasets.append(dataset)
         for dataset in datasets:
             getattr(new_ws, "import")(dataset)
             self.stdout.info(f"INFO: Imported dataset \"{dataset.GetName()}\".")
