@@ -17,7 +17,7 @@ class ParameterisedAsymptoticCLs(ParameterisedRunner):
     def __init__(self, input_path:str, config:Optional[Dict]=None,
                  file_expr:Optional[str]=None, param_expr:Optional[str]=None,
                  filter_expr:Optional[str]=None, exclude_expr:Optional[str]=None,
-                 outdir:str="output", outname:str="limits.json",
+                 outdir:str="output", cachedir:str="cache", outname:str="limits.json",
                  save_summary:bool=False, cache:bool=True,
                  save_log:bool=True, parallel:int=-1,
                  verbosity:Optional[Union[int, str]]="INFO"):
@@ -31,6 +31,7 @@ class ParameterisedAsymptoticCLs(ParameterisedRunner):
             'input_path': input_path,
             'config': config,
             'outdir': outdir,
+            'cachedir': cachedir,
             'outname': outname,
             'save_summary': save_summary
         }
@@ -72,7 +73,7 @@ class ParameterisedAsymptoticCLs(ParameterisedRunner):
             return {}
         
     def get_cache_dir(self):
-        return os.path.join(self.attributes['outdir'], 'cache')
+        return os.path.join(self.attributes['outdir'], self.attributes['cachedir'])
         
     def prepare_task_inputs(self):
         input_path = self.attributes['input_path']
