@@ -58,10 +58,7 @@ def get_root_version():
     return root_version
 
 def get_workspace_extensions():
-    resource_path = quickstats.resource_path
-    extension_config_file = os.path.join(resource_path, "workspace_extensions.json")
-    with open(extension_config_file, "r") as file:
-        extension_config = json.load(file)
+    extension_config = get_workspace_extension_config()
     extensions = extension_config['required']
     if not extension_config['strict']:
         # deprecated extension
@@ -69,7 +66,17 @@ def get_workspace_extensions():
             extensions.remove('FlexibleInterpVarMkII')
     return extensions
 
-def add_macro(path:str, name:Optional[str]=None, copy_files:bool=True, workspace_extension:bool=True):
+def get_workspace_extension_config():
+    resource_path = quickstats.resource_path
+    extension_config_file = os.path.join(resource_path, "workspace_extensions.json")
+    if not os.path.exists(extension_config_file):
+        extension_config_file = os.path.join(resource_path, "default_workspace_extensions.json")
+    with open(extension_config_file, "r") as file:
+        extension_config = json.load(file)
+    return extension_config
+
+def add_macro(path:str, name:Optional[str]=None, copy_files:bool=True,
+              workspace_extension:bool=True, force:bool=False):
     if not os.path.isdir(path):
         raise ValueError("macro path must be a directory")
     if name is None:
@@ -81,8 +88,7 @@ def add_macro(path:str, name:Optional[str]=None, copy_files:bool=True, workspace
     if copy_files:
         dest_path = os.path.join(macro_path, name)
         if os.path.exists(dest_path):
-            answer = input(f'WARNING: The macro already exists in {dest_path}, overwrite? [Y/N]')
-            if answer == "Y":
+            if force or input(f'WARNING: The macro already exists in {dest_path}, overwrite? [Y/N]') == "Y":
                 shutil.rmtree(dest_path)
                 shutil.copytree(path, dest_path)
                 quickstats._PRINT_.info(f'INFO: Overwritten contents for the macro "{name}" from {path} to {dest_path}.')
@@ -94,35 +100,32 @@ def add_macro(path:str, name:Optional[str]=None, copy_files:bool=True, workspace
     if workspace_extension:
         resource_path = quickstats.resource_path
         extension_config_file = os.path.join(resource_path, "workspace_extensions.json")
-        with open(extension_config_file, "r") as file:
-            extension_config = json.load(file)
+        extension_config = get_workspace_extension_config()
         if name not in extension_config['required']:
             extension_config['required'].append(name)
             with open(extension_config_file, "w") as file:
                 json.dump(extension_config, file, indent=2)
         quickstats._PRINT_.info(f'INFO: The macro "{name}" has been added to the workspace extension list.')
         
-def remove_macro(name:str, remove_files:bool=True):
+def remove_macro(name:str, remove_files:bool=True, force:bool=False):
     resource_path = quickstats.resource_path
     extension_config_file = os.path.join(resource_path, "workspace_extensions.json")
-    with open(extension_config_file, "r") as file:
-        extension_config = json.load(file)
+    extension_config = get_workspace_extension_config()
     if (name not in extension_config):
-        quickstats.__PRINT__.info(f'WARNING: Extension "{name}" not found in the workspace extension list. Skipped.')
+        quickstats._PRINT_.info(f'WARNING: Extension "{name}" not found in the workspace extension list. Skipped.')
     else:
         extension_config.remove(name)
         with open(extension_config_file, "w") as file:
             json.dump(extension_config, file, indent=2)
-        quickstats.__PRINT__.info(f'INFO: The extension "{name}" has been removed from the workspace extension list.')
+        quickstats._PRINT_.info(f'INFO: The extension "{name}" has been removed from the workspace extension list.')
     if remove_files:
         macro_path = quickstats.macro_path
         extension_path = os.path.abspath(os.path.join(macro_path, name))
-        answer = input(f'WARNING: Attempting to remove files from {extension_path}, confirm? [Y/N]')
-        if answer == "Y":
-            quickstats._PRINT_.info(f'INFO: Removal confirmed.')
+        if force or input(f'WARNING: Attempting to remove files from {extension_path}, confirm? [Y/N]') == "Y":
+            quickstats._PRINT_.info(f'INFO: Files for the macro "{name}" has been removed from {extension_path}.')
             shutil.rmtree(extension_path)
         else:
-            quickstats._PRINT_.info(f'INFO: Removal cancelled.')
+            quickstats._PRINT_.info(f'INFO: Macro files removal cancelled.')
 
 def load_corelib():
     if not quickstats.corelib_loaded:
