@@ -1,5 +1,6 @@
 from typing import List
 import cppyy
+import cppyy.ll
 import ctypes
 import numpy as np
 
@@ -110,7 +111,6 @@ def as_c_array(data:np.ndarray):
     if c_type is None:
         raise ValueError(f"unsupported data type \"{data.dtype}\"")
     c_type_p = ctypes.POINTER(c_type)
-    c_type_str = ctype_str_maps[c_type]
     c_data = data.ctypes.data_as(c_type_p)
     return c_data
 
@@ -119,6 +119,12 @@ def as_np_array(vec:cppyy.gbl.std.vector):
         return np.array([str(v) for v in vec])
     return np.array(vec.data())
 
+def vector_to_pointer(data:cppyy.gbl.std.vector):
+    return cppyy.ll.cast[f'{data.value_type}*'](cppyy.gbl.VecUtils.as_pointer(data))
+
+def pointer_to_vector(data:"cppyy.LowLevelView", size:int):
+    return cppyy.gbl.VecUtils.as_vector(data, size)
+
 def c_array_to_np_array(arr:"cppyy.LowLevelView", size:int, shape=None, copy:bool=True):
     if not hasattr(arr, 'typecode'):
         raise RuntimeError(f"input must be an instance of cppyy.LowLevelView")
