@@ -242,4 +242,96 @@ class RooDataSet:
             ind = np.argsort(bin_value)
             bin_weight = data['weight'][mask][ind]
             result[cat_name] = bin_weight
-        return result
\ No newline at end of file
+        return result
+    
+    @staticmethod
+    def fill_from_TH1(dataset:"ROOT.RooDataSet", hist:"ROOT.TH1",
+                      skip_out_of_range:bool=True,
+                      blind_range:Optional[List[float]]=None,
+                      min_bin_value:float=0,
+                      weight_scale:float=1):
+        import ROOT
+        parameters = dataset.get()
+        if parameters.size() > 1:
+            raise RuntimeError("multiple observables are not allowed")
+        x = parameters.first()
+        weight_var = dataset.weightVar()
+        # blinding will be taken care of
+        xmin = x.getMin()
+        xmax = x.getMax()
+        n_bins = hist.GetNbinsX()
+        for i in range(1, n_bins + 1):
+            bin_center = hist.GetBinCenter(i)
+            # skip bins that are out of range
+            if skip_out_of_range and ((bin_center > xmax) or (bin_center < xmin)):
+                continue
+            # skip bins in the blind range
+            if (blind_range and (bin_center > blind_range[0]) and (bin_center < blind_range[1])):
+                continue
+            x.setVal(bin_center)
+            bin_content = hist.GetBinContent(i)
+            weight = bin_content * weight_scale
+            # if the weight is negligible, consider it as zero
+            if (weight < min_bin_value):
+                continue;
+            if weight_var:
+                weight_var.setVal(weight)
+                dataset.add(ROOT.RooArgSet(x, weight_var), weight)
+            else:
+                dataset.add(ROOT.RooArgSet(x), weight)
+    
+    @staticmethod
+    def get_x_and_weight(dataset:"ROOT.RooDataSet"):
+        parameters = dataset.get()
+        if parameters.size() > 1:
+            raise RuntimeError("multiple observables are not allowed")
+        x = parameters.first()
+        weight_var = dataset.weightVar()
+        return x, weight_var
+    
+    @staticmethod
+    def to_TH1(dataset:"ROOT.RooDataSet", name:str,
+               blind_range:Optional[List[float]]=None,
+               weight_scale:float=1):
+        x, weight_var = RooDataSet.get_x_and_weight(dataset)
+        n_bins = x.getBins()
+        x_min = x.getMin()
+        x_max = x.getMax()
+        import ROOT
+        hist = ROOT.TH1D(name, name, n_bins, x_min, x_max)
+        hist.Sumw2()
+        for i in range(dataset.numEntries()):
+            dataset.get(i)
+            x_val = x.getVal()
+            obs.setVal(x_val)
+            weight = dataset.weight() * weight_scale
+            # ignore data in the blind range
+            if (blind_range and (x_val > blind_range[0]) and (x_val < blind_range[1])):
+                continue
+            hist.Fill(x_val, weight)
+        return hist
+    
+    @staticmethod
+    def add_ghost_weights(dataset:"ROOT.RooDataSet",
+                          blind_range:Optional[List[float]]=None,
+                          ghost_weight:float=1e-9):
+        x, weight_var = RooDataSet.get_x_and_weight(dataset)
+        xmin = x.getMin()
+        xmax = x.getMax()
+        n_bins = x.getBins()
+        bin_width = (xmax - xmin) / n_bins
+        data = RooDataSet.to_numpy(dataset)
+        x_data = data[x.GetName()]
+        weight_data = data["weight"]
+        hist, bin_edges = np.histogram(x_data, bins=n_bins, range=(xmin, xmax), weights=weight_data)
+        from quickstats.maths.statistics import bin_edge_to_bin_center
+        bin_centers = bin_edge_to_bin_center(bin_edges)
+        import ROOT
+        for bin_val, bin_center in zip(hist, bin_centers):
+            if (bin_val != 0):
+                continue
+            if (blind_range and (bin_center > blind_range[0]) and (bin_center < blind_range[1])):
+                continue
+            x.setVal(bin_center)
+            weight_var.setVal(ghost_weight)
+            dataset.add(ROOT.RooArgSet(x, weight_var), ghost_weight)
\ No newline at end of file
