@@ -92,13 +92,49 @@ ROOT_MACROS = \
             std::vector<T> result;
             result.reserve(n_bin);
             const size_t bin_min = 1 - underflow_bin;
-            const size_t bin_max = n_bin + overflow_bin;
+            // the + 1 is because number of bin low edges = number of bins + 1
+            const size_t bin_max = n_bin + overflow_bin + 1;
             for (size_t bin_index = bin_min; bin_index <= bin_max; bin_index++)
                 result.push_back(h->GetBinLowEdge(bin_index));
             return result;
-        }    
+        }
+        
+        template<typename T>
+        int FindBinIndexByBinEdge(TH1* h, const T &bin_edge, const double &epsilon=1e-6)
+        {
+            const T first_edge = h->GetBinLowEdge(1);
+            if ((bin_edge < first_edge) && abs(bin_edge - first_edge) > epsilon)
+                return 0;
+            auto n_bins = h->GetNbinsX();
+            const T last_edge = h->GetBinLowEdge(n_bins);
+            if ((bin_edge > last_edge) && abs(bin_edge - last_edge) > epsilon)
+                return n_bins + 1;
+            for (size_t i = 1; i < n_bins + 1; i++){
+                const T edge_i = h->GetBinLowEdge(i);
+                if (abs(bin_edge - edge_i) < epsilon)
+                    return i;
+            }
+            return -1;
+        }
     }; 
     """,
+    "TF1Utils":
+    """
+    namespace TF1Utils {
+        std::vector<double> GetRandomArray(TF1* f, const int &size, const double &xmin, const double &xmax){
+            std::vector<double> result(size);
+            for (size_t i = 0; i < size; i++)
+                result[i] = f->GetRandom(xmin, xmax);
+            return result;
+        }
+        std::vector<double> GetRandomArray(TF1* f, const int &size){
+            std::vector<double> result(size);
+            for (size_t i = 0; i < size; i++)
+                result[i] = f->GetRandom();
+            return result;
+        }
+    }; 
+    """,    
     "TAxisUtils": 
     """
     namespace TAxisUtils{
@@ -230,6 +266,9 @@ ROOT_MACROS = \
             }
           }
         } ;
+        
+        #if ROOT_VERSION_CODE >= ROOT_VERSION(6,26,0)
+        #endif
     };
     """
 }
