@@ -1,16 +1,17 @@
-from typing import Optional, Union, Dict, List
+from typing import Optional, Union, Dict, List, Callable
 import os
 import re
 import copy
 import glob
 import fnmatch
 import itertools
+from functools import partial
 
 import numpy as np
 
-from quickstats import semistaticmethod
-from quickstats.maths.numerics import pretty_float, str_encode_value, str_decode_value
-
+from quickstats import semistaticmethod, DescriptiveEnum
+from quickstats.maths.numerics import pretty_float, str_encode_value, str_decode_value, to_string, to_rounded_float
+from quickstats.utils.string_utils import remove_whitespace
 
 signature_regex = {
     'F': r"\d+[.]?\d*",
@@ -36,6 +37,18 @@ signature_parser = {
     'S': str
 }
 
+class MatchMode(DescriptiveEnum):
+    
+    Numerical = (0, "Match by value", lambda x, y: to_rounded_float(x) == to_rounded_float(y))
+    Wildcard  = (1, "Match by wildcard", lambda x, y: fnmatch.fnmatch(to_string(x), to_string(y)))
+
+    def __new__(cls, value:int, description:str, match_func:Callable):
+        obj = object.__new__(cls)
+        obj._value_ = value
+        obj.description = description
+        obj.match_func = match_func
+        return obj
+
 class ParamParser:
     
     DEFAULT_FORMAT_STR = r"[\w-]+"
@@ -267,35 +280,50 @@ class ParamParser:
         return selected_param_points
     
     @staticmethod
-    def parse_filter_expr(expr:Optional[str]=None):
+    def parse_filter_expr(expr:str):
         if expr is None:
-            return {}
+            return []
+        expr = remove_whitespace(expr)
+        match_conditions = []
         tokens = expr.split(";")
-        conditions = {}
         for token in tokens:
-            subtokens = token.split("=")
-            if len(subtokens) != 2:
-                raise RuntimEerror(f"invalid filter/exclude expression `{expr}`")
-            param_name = subtokens[0].strip()
-            param_conditions = [s.strip() for s in subtokens[1].split(",")]
-            conditions[param_name] = param_conditions
-        return conditions
+            match_condition = {}
+            param_conditions = token.split(",")
+            for param_condition in param_conditions:
+                subtokens = param_condition.split("=")
+                if len(subtokens) != 2:
+                    raise RuntimeError(f"invalid parameter condition `{param_condition}` in the "
+                                       f"filter/exclude expression `{expr}`")
+                param_name, condition = subtokens
+                if "*" in condition:
+                    match_condition[param_name] = partial(MatchMode.Wildcard.match_func, y=condition)
+                else:
+                    match_condition[param_name] = partial(MatchMode.Numerical.match_func, y=condition)
+            match_conditions.append(match_condition)
+        return match_conditions
     
     @staticmethod
-    def select_param_point(parameters, conditions:Dict, default:bool=True):
-        selected = {}
-        for param_name in conditions:
-            if param_name not in parameters:
-                selected[param_name] = default
-                continue
-            param_value = parameters[param_name]
-            if isinstance(param_value, float):
-                param_value = round(param_value, 8)
-            param_value = str(param_value)
-            param_conditions = conditions[param_name]
-            selected[param_name] = any([fnmatch.fnmatch(param_value, cond) for cond in param_conditions])
-        selected = all([v for v in selected.values()])
-        return selected
+    def select_param_point(parameters, conditions:List[Dict], default:bool=True):
+        selected = not default
+        for condition in conditions:
+            for param_name, match_func in condition.items():
+                if param_name not in parameters:
+                    continue
+                param_value = parameters[param_name]
+                if not match_func(param_value):
+                    return not default
+        return default
+    
+    @staticmethod
+    def select_param_point(parameters, conditions:List[Dict], default:bool=True):
+        for condition in conditions:
+            for param_name, match_func in condition.items():
+                # consider not matched if any of the param-specific condition not satisfied
+                if (param_name not in parameters) or (not match_func(parameters[param_name])):
+                    break
+            else:
+                return default
+        return not default
     
     def select_param_points(self, param_points:List,
                             filter_expr:Optional[str]=None,
