@@ -1,5 +1,6 @@
 import quickstats
 
+from .core import *
 from .color_schemes import *
 from .abstract_plot import AbstractPlot
 from .collective_data_plot import CollectiveDataPlot
@@ -18,6 +19,7 @@ from .likelihood_2D_plot import Likelihood2DPlot
 from .pdf_distribution_plot import PdfDistributionPlot
 from .correlation_plot import CorrelationPlot
 from .histo_1D_plot import Histo1DPlot
+from .sample_purity_plot import SamplePurityPlot
 
 from matplotlib import style, colors
 
@@ -29,3 +31,5 @@ style.use("quick_default")
 
 colors.EXTRA_COLORS = EXTRA_COLORS
 colors.colorConverter.colors.update(EXTRA_COLORS)
+
+register_cmaps(QUICKSTATS_PALETTES)
\ No newline at end of file
