@@ -98,6 +98,9 @@ TEMPLATE_STYLES = {
         'ratio_frames':{
             'height_ratios': (3, 1),
             'hspace': 0.07            
+        },
+        'barh': {
+            'height': 0.5
         }
     }
 }
@@ -232,7 +235,7 @@ def suggest_markersize(nbins:int):
     return size_min
 
 def format_axis_ticks(ax, x_axis=True, y_axis=True, major_length:int=16, minor_length:int=8,
-                      spine_width:int=2, major_width:int=2, minor_width:int=1, 
+                      spine_width:int=2, major_width:int=2, minor_width:int=1, direction:str='in',
                       label_bothsides:bool=False, tick_bothsides:bool=False,
                       labelsize:Optional[int]=None,
                       offsetlabelsize:Optional[int]=None,
@@ -248,11 +251,12 @@ def format_axis_ticks(ax, x_axis=True, y_axis=True, major_length:int=16, minor_l
         styles['labelbottom'] = True
         styles['top'] = tick_bothsides
         styles['bottom'] = True
+        styles['direction'] = direction
         if x_axis_styles is not None:
             styles.update(x_axis_styles)
-        ax.tick_params(axis="x", which="major", direction='in', length=major_length,
+        ax.tick_params(axis="x", which="major", length=major_length,
                        width=major_width, **styles)
-        ax.tick_params(axis="x", which="minor", direction='in', length=minor_length,
+        ax.tick_params(axis="x", which="minor", length=minor_length,
                        width=minor_width, **styles)
     if y_axis:
         if (ax.get_yaxis().get_scale() != 'log'):
@@ -262,11 +266,12 @@ def format_axis_ticks(ax, x_axis=True, y_axis=True, major_length:int=16, minor_l
         styles['labelright'] = label_bothsides
         styles['left'] = True
         styles['right'] = tick_bothsides
+        styles['direction'] = direction
         if y_axis_styles is not None:
             styles.update(y_axis_styles)
-        ax.tick_params(axis="y", which="major", direction='in', length=major_length,
+        ax.tick_params(axis="y", which="major", length=major_length,
                        width=major_width, **styles)
-        ax.tick_params(axis="y", which="minor", direction='in', length=minor_length,
+        ax.tick_params(axis="y", which="minor", length=minor_length,
                        width=minor_width, **styles)
         
     for axis in ['top','bottom','left','right']:
