@@ -252,13 +252,20 @@ def set_unlimited_stacksize():
     import resource
     resource.setrlimit(resource.RLIMIT_STACK, (resource.RLIM_INFINITY, resource.RLIM_INFINITY))
     
-def in_notebook():
+def in_notebook() -> bool:
     try:
         from IPython import get_ipython
-        if 'IPKernelApp' not in get_ipython().config:  # pragma: no cover
-            return False
+        shell = get_ipython().__class__.__name__
+        if shell == 'ZMQInteractiveShell':
+            return True   # Jupyter notebook or qtconsole
+        elif shell == 'TerminalInteractiveShell':
+            return False  # Terminal running IPython
+        else:
+            return False  # Other type (?)
     except ImportError:
         return False
     except AttributeError:
         return False
+    except NameError:
+        return False      # Probably standard Python interpreter    
     return True
\ No newline at end of file
